%**** FUNCTION COMPUTERHS.M ********************************
%                                                          *
%  S R Lau                                                 *
%                                                          *
%  Applied Mathematics Group                  1998 to 2004 *
%  University of North Carolina                            *
%  Chapel Hill, NC 27599                                   *
%                                                          *
%  Center for Gravitational Wave Astronomy    2004 to 2005 *
%  University of Texas at Brownsville                      *
%  Brownsville, TX 78520                                   *
%                                                          *
%  Mathematics & Statistics                   2005 to 2006 *
%  University of New Mexico                                *
%  Albuquerque, NM 87131                                   *
%                                                          *
%  Division of Applied Mathematics            2006 to 2009 *
%  Brown University                                        *
%  Providence, RI 02906                                    *
%                                                          *
%  Last modified: 21 December 2012                         *
%                                                          *
%  Note W = -Pi - Phi = (d/dt - d/dx)psi  (propagates -->) *
%       X = -Pi + Phi = (d/dt + d/dx)psi  (propagates <--) *
%                                                          *
%***********************************************************
      function [fpsi,fPi,fPhi]=ComputeRhs(terk,psi,Pi,Phi,Vee,Wa,Xb);
      npts = length(psi(:,1));
      ndmn = length(psi(1,:));
      %
      %  BC at inner boundary x = a; Xa fixed from 
      %  Pi and Phi but Wa comes from passed in value.
      %
      q =  1;
      Xa          = -Pi(1,q)+Phi(1,q);
      Pi(1,q)     = -0.5*(Xa+Wa);
      Phi(1,q)    =  0.5*(Xa-Wa);
      %
      %  BC at outer boundary x = b; Wb fixed from
      %  Pi and Phi but Xb comes from passed in value.
      %
      q = ndmn;
      Wb          = -Pi(npts,q)-Phi(npts,q);
      Pi(npts,q)  = -0.5*(Xb+Wb);
      Phi(npts,q) =  0.5*(Xb-Wb);
      %
      %  Passing of characteristic information 
      %  at ALL internal boundaries x = c.
      %
      for q = 2:ndmn
          Wc            = -Pi(npts,q-1) - Phi(npts,q-1);
          Xc            = -Pi(1,q) + Phi(1,q);
          Pi(npts,q-1)  = -0.5*(Xc+Wc);
          Phi(npts,q-1) =  0.5*(Xc-Wc);
          Pi(1,q)       = -0.5*(Xc+Wc);
          Phi(1,q)      =  0.5*(Xc-Wc);
      end
%
%  Main part of routine where time derivatives are computed.
%
      GlobalArrays
      DPi  =  D*Pi;
      DPhi =  D*Phi;
      fpsi = -Pi;
      fPi  = -DPhi + Vee.*psi;
      fPhi = -DPi;
