%**** SCRIPT ERK_SCHST_CHB_MULT_EXTR.m *********************
%                                                          *
%  S R Lau                                                 *
%                                                          *
%  Applied Mathematics Group                  1998 to 2004 *
%  University of North Carolina                            *
%  Chapel Hill, NC 27599                                   *
%                                                          *
%  Center for Gravitational Wave Astronomy    2004 to 2005 *
%  University of Texas at Brownsville                      *
%  Brownsville, TX 78520                                   *
%                                                          *
%  Mathematics & Statistics                   2005 to 2006 *
%  University of New Mexico                   2009 to ???? *
%  Albuquerque, NM 87131                                   *
%                                                          *
%  Division of Applied Mathematics            2006 to 2008 *
%  Brown University                                        *
%  Providence, RI 02906                                    *
%                                                          *
%  Last modified: 21 December 2012                         *
%  ERK = Explicit Runge Kutta                              *
%  SCHST = SCHwarzschild Static Time slices                *
%  CHB = CHeByshev                                         *
%  MULT = MULTidomain                                      *
%  EXTR = EXTRaction                                       *
%                                                          *
%***********************************************************
      GlobalArrays
%***********************************************************
%
%  Tag variables are set in Experiment scripts.
%
      printTee     =  printTee_tag;  % Print time in steps of printTee.
      makePlots    =  makePlots_tag; % Decide whether to plot midflight.
      MAXTIME      =  MAXTIME_tag;
      npts         =  npts_tag;
      exponents    =  exponents_tag;
      mass         =  mass_tag;
      ndmn         =  ndmn_tag;
      a            =  a_tag;
      rhoBd        =  rhoBd_tag;
      rhoBd0       =  rhoBd0_tag;
      rhoBd2       =  rhoBd2_tag;
%***********************************************************
      %
      %  For no driving particle RW and CPM are the same thing.
      %  model 0(Regge-Wheeler)
      %        1(Cunningham-Price-Moncrief)
      %        2(Zerilli-Moncrief)
      %
      model        = 0;
      ell          = 2;
      bsch         = 2*mass*rhoBd;
      Fsch         = 1-2*mass/bsch;
      b            = bsch+2*mass*log(0.5*bsch/mass-1);
      Ddmn         = (b-a)/ndmn;
      c(1:ndmn+1)  = [a:Ddmn:b];
      nsub         = npts*ones(1,ndmn);
      n1           = [1,1+cumsum(nsub(1:ndmn-1))];
      n2           = cumsum(nsub);
      Dref         = chebD(npts-1);
%
%  Butcher table for RK4. Can swap out for another here.
%
      nerk = 4; % Number of Explicit RK stages.
      erk_a   = zeros(nerk,nerk);
      erk_b   = zeros(1,nerk);
      erk_bhat = zeros(1,nerk);
      erk_c   = zeros(1,nerk);
      erk_a(1,:) = [0.0,     0.0,     0.0,     0.0    ];
      erk_a(2,:) = [0.5,     0.0,     0.0,     0.0    ];
      erk_a(3,:) = [0.0,     0.5,     0.0,     0.0    ];
      erk_a(4,:) = [0.0,     0.0,     1.0,     0.0    ];
      erk_c      = [0.0,     0.5,     0.5,     1.0    ];
      erk_b      = [1.0/6.0, 1.0/3.0, 1.0/3.0, 1.0/6.0];
%
%  Arrays for coordinates, potential, solution, and derivatives. 
%
      x       = zeros(npts,ndmn); 
      z       = zeros(npts,ndmn);
      Vee     = zeros(npts,ndmn);
      psi     = zeros(npts,ndmn);
      Pi      = zeros(npts,ndmn);
      Phi     = zeros(npts,ndmn);
      DPi     = zeros(npts,ndmn);
      DPhi    = zeros(npts,ndmn);
%
%  Individual domains with Chebyshev-Gauss-Lobatto nodes.
%
      for q = 1:ndmn
          z(:,q) = cos(pi*(nsub(q)-1:-1:0) ...
                         / (nsub(q)-1));
          x(:,q) = 0.5*(c(q+1)-c(q)) * z(:,q) ...
                         + 0.5*(c(q)+c(q+1)) ...
                         * ones(size(z(:,q)));
      end
%
%  Differentiation matrix for all subdomains.
%
      D = -(2.0/(x(npts,1)-x(1,1)))*Dref;
%
%  Get the potential.
%
      switch model
         case {0,1}, 
            spin = 2;
            Vee=ReggeWheelerV(x,mass,ell,spin);
         case 2,
            Vee=ZerilliV(x,mass,ell);
         otherwise,
            error = 'model not 0 1 or 2.'
         pause
      end
%
%  Uniform reference grid. [a0,b0] may or may not be [a,b].
% 
      n0    = 1024;
      a0    = a;
      bsch0 = 2*mass*rhoBd0;
      b0    = bsch0+2*mass*log(0.5*bsch0/mass-1);
      x0    = linspace(a0,b0,n0)';
      psi0  = zeros(size(x0));
      Pi0   = zeros(size(x0));
      Phi0  = zeros(size(x0));
%
%  Arrays to record time steps and errors.
%
      plotDt  = zeros(length(exponents)-1,1);
      plotErr = zeros(length(exponents)-1,3);
%
%  Loop over solves of varying temporal coarseness.
%
      qtot = length(exponents);
      for qexp = 1:qtot;
         n	= round(2^exponents(qexp));
         DTee   = MAXTIME/n;
         nsteps = round(MAXTIME/DTee); 
         output = zeros(nsteps+1,4);
         disp([' '])
         disp([' '])
         disp([' '])
         disp(['Resolution ',num2str(qexp),' of ',num2str(qtot)])
         disp(['-----------------------------------------'])
         disp(['DTee               : ',num2str(DTee)])
         %
         %  Initial Data.
         %
         Tee      = 0;
         xcenter = 4*mass; sig = 2.0;
         for k = 1:ndmn
             [psi(:,k), Phi(:,k)]=ExpBumpData(x(:,k),xcenter,sig,mass);
             Pi(:,k)  = Phi(:,k);
         end
         if makePlots == 1
%
%  Interpolate initial data onto reference grid.
%
         for i = 1:length(x0);
             for q = 1:ndmn
                 if x0(i) >= c(q) & x0(i) <= c(q+1)
                    psi0(i) = chbevalv(real(psi(:,q)), ...
                                     x0(i),c(q),c(q+1));
                    Pi0(i) = chbevalv(real(Pi(:,q)), ...
                                     x0(i),c(q),c(q+1));
                    Phi0(i) = chbevalv(real(Phi(:,q)), ...
                                     x0(i),c(q),c(q+1));
                    break
                 end
             end
         end
%
%  Plot initial psi, Pi, Phi.
%
         figure(1)
         clf
         subplot(3,1,1)
         hold off
         plot(x0,psi0,'b','LineWidth',2)
         set(gca, 'FontSize', 16, 'fontweight', 'n')
         legend('Initial','Location','NorthWest')
         title('Solution $\psi$','interpreter','latex')
         subplot(3,1,2)
         hold off
         plot(x0,Pi0,'b','LineWidth',2)
         set(gca, 'FontSize', 16, 'fontweight', 'n')
         legend('Initial','Location','NorthWest')
         title('Solution $\Pi$','interpreter','latex')
         subplot(3,1,3)
         hold off
         plot(x0,Phi0,'b','LineWidth',2)
         set(gca, 'FontSize', 16, 'fontweight', 'n')
         legend('Initial','Location','NorthWest')
         title('Solution $\Phi$','interpreter','latex')
         xlabel('$x$','interpreter','latex')
         end
%
%  Initialize run variables.
%  Note W = -Pi - Phi = (d/dt - d/dx)psi  (propagates -->) 
%       X = -Pi + Phi = (d/dt + d/dx)psi  (propagates <--) 
%  so Wa at x=a and Xb at x=b need fixing. Set to zero for now.
%
         Tee      = 0;
         OrbitTee = 0;
         Wa       = 0;
         Xb       = 0;
         fpsi     = zeros(npts,ndmn,nerk);
         fPi      = zeros(npts,ndmn,nerk);
         fPhi     = zeros(npts,ndmn,nerk);
         psik     = zeros(npts,ndmn,nerk);
         Pik      = zeros(npts,ndmn,nerk);
         Phik     = zeros(npts,ndmn,nerk);
%
%  Variables needed for radiation boundary conditions. Normally we use Matlab
%  ready tables like FdCqRho0015L002D19_rw2, but here load in the actual
%  tables (e.g. SumOfPolesRBC-Table.txtLOAD) in arXiv:1210.1565 as a check.
%
         if rhoBd == 15
            Table=load('SumOfPolesRBC-Table.txtLOAD');
            dsing = 17; dpair = 1;
            kap = Table(1:dsing,3);
            mu  = Table(1:dsing,1);
            qR  = Table(dsing+1:dsing+dpair,3);
            qI  = Table(dsing+1:dsing+dpair,4);
            mR  = Table(dsing+1:dsing+dpair,1);
            mI  = Table(dsing+1:dsing+dpair,2);
%%%%%
%%%%%   MATLAB ready table (should be the same)
%%%%%   [kap,mu,qR,qI,mR,mI]=FdCqRho0015L002D19_rw2;
%%%%%
            kap = kap/(2*mass);
            mu = mu/(2*mass);
            qR = qR/(2*mass);
            qI = qI/(2*mass);
            mR = mR/(2*mass);
            mI = mI/(2*mass);
         else
            %
            %  Will be Sommerfeld RBC since mu,mR,mI all zero.
            %
            mu  =  0; mR =  0; mI =  0;
            kap = -1; qR = -1; qI = -1;
         end
         nmu = length(mu);
         nmR = length(mR);
         Hstarpsi  = zeros(nmu,1);
         Gstarpsi  = zeros(nmR,1);
         Fstarpsi  = zeros(nmR,1);
         Hstarpsik = zeros(nmu,nerk);
         Gstarpsik = zeros(nmR,nerk);
         Fstarpsik = zeros(nmR,nerk);
         fHstarpsi = zeros(nmu,nerk);
         fGstarpsi = zeros(nmR,nerk);
         fFstarpsi = zeros(nmR,nerk);
%
%  The underscore _E here means "Extraction".
%
         if rhoBd2 ~= rhoBd
            Table=load('SumOfPolesEXT-TableLong.txtLOAD');
            dsing = 24; dpair = 1;
            kap_E = Table(1:dsing,3);
            mu_E  = Table(1:dsing,1);
            qR_E  = Table(dsing+1:dsing+dpair,3);
            qI_E  = Table(dsing+1:dsing+dpair,4);
            mR_E  = Table(dsing+1:dsing+dpair,1);
            mI_E  = Table(dsing+1:dsing+dpair,2);
%%%%%
%%%%%       MATLAB ready table (should be the same)
%%%%%       [kap_E,mu_E,qR_E,qI_E,mR_E,mI_E]=FdCqRho0015to1E15L002D26_rw2;
%%%%%
            kap_E = kap_E/(2*mass);
            mu_E  = mu_E/(2*mass);
            qR_E  = qR_E/(2*mass);
            qI_E  = qI_E/(2*mass);
            mR_E  = mR_E/(2*mass);
            mI_E  = mI_E/(2*mass);
         else
            mu_E  =  0; mR_E =  0; mI_E =  0;
            kap_E = -1; qR_E = -1; qI_E = -1;
         end
         nmu_E = length(mu_E);
         nmR_E = length(mR_E);
         Hstarpsi_E  = zeros(nmu_E,1);
         Gstarpsi_E  = zeros(nmR_E,1);
         Fstarpsi_E  = zeros(nmR_E,1);
         Hstarpsik_E = zeros(nmu_E,nerk);
         Gstarpsik_E = zeros(nmR_E,nerk);
         Fstarpsik_E = zeros(nmR_E,nerk);
         fHstarpsi_E = zeros(nmu_E,nerk);
         fGstarpsi_E = zeros(nmR_E,nerk);
         fFstarpsi_E = zeros(nmR_E,nerk);
%***********************************************************
%***********************************************************
%***********************************************************
         tic
         for j = 1:nsteps,
             for s = 1:nerk
                 terk       = Tee+DTee*erk_c(s);
                 psik(:,:,s)    = psi;
                 Pik(:,:,s)     = Pi;
                 Phik(:,:,s)    = Phi;
                 Hstarpsik(:,s) = Hstarpsi;
                 Gstarpsik(:,s) = Gstarpsi;
                 Fstarpsik(:,s) = Fstarpsi;
                 Hstarpsik_E(:,s) = Hstarpsi_E;
                 Gstarpsik_E(:,s) = Gstarpsi_E;
                 Fstarpsik_E(:,s) = Fstarpsi_E;
                 for k=1:s-1
                    if erk_a(s,k) ~= 0.0
                       psik(:,:,s) = psik(:,:,s) ...
                               + erk_a(s,k)*DTee*fpsi(:,:,k);
                       Pik(:,:,s) = Pik(:,:,s) ...
                               + erk_a(s,k)*DTee*fPi(:,:,k);
                       Phik(:,:,s) = Phik(:,:,s) ...
                               + erk_a(s,k)*DTee*fPhi(:,:,k);
                       Hstarpsik(:,s) = Hstarpsik(:,s) ...
                               + erk_a(s,k)*DTee*fHstarpsi(:,k);
                       Gstarpsik(:,s) = Gstarpsik(:,s) ...
                               + erk_a(s,k)*DTee*fGstarpsi(:,k);
                       Fstarpsik(:,s) = Fstarpsik(:,s) ...
                               + erk_a(s,k)*DTee*fFstarpsi(:,k);
                       Hstarpsik_E(:,s) = Hstarpsik_E(:,s) ...
                               + erk_a(s,k)*DTee*fHstarpsi_E(:,k);
                       Gstarpsik_E(:,s) = Gstarpsik_E(:,s) ...
                               + erk_a(s,k)*DTee*fGstarpsi_E(:,k);
                       Fstarpsik_E(:,s) = Fstarpsik_E(:,s) ...
                               + erk_a(s,k)*DTee*fFstarpsi_E(:,k);
                    end
                 end
                 Xb = (Fsch/bsch)*(  sum(mu.*Hstarpsik(:,s)) ...
                                  +2*sum(mR.*Gstarpsik(:,s)) ...
                                  -2*sum(mI.*Fstarpsik(:,s)));
                 %
                 %  Uncomment to take out convolution RBC and 
                 %  use simple Sommerfeld boundary condition.
                 %  Xb = 0;
                 %
                 [fpsi(:,:,s),fPi(:,:,s),fPhi(:,:,s)] = ... 
                 ComputeRhs(terk,psik(:,:,s),Pik(:,:,s), ...
                    Phik(:,:,s),Vee,Wa,Xb);
                 %
                 fHstarpsi(:,s) = kap.*Hstarpsik(:,s) ...
                                + psik(npts,ndmn,s);
                 fGstarpsi(:,s) = qR.*Gstarpsik(:,s) ...
                                - qI.*Fstarpsik(:,s) ...
                                + psik(npts,ndmn,s);
                 fFstarpsi(:,s) = qR.*Fstarpsik(:,s) ...
                                + qI.*Gstarpsik(:,s);
                 %
                 fHstarpsi_E(:,s) = kap_E.*Hstarpsik_E(:,s) ...
                                  + psik(npts,ndmn,s);
                 fGstarpsi_E(:,s) = qR_E.*Gstarpsik_E(:,s) ...
                                  - qI_E.*Fstarpsik_E(:,s) ...
                                  + psik(npts,ndmn,s);
                 fFstarpsi_E(:,s) = qR_E.*Fstarpsik_E(:,s) ...
                                  + qI_E.*Gstarpsik_E(:,s);
             end
             for s = 1:nerk
                 psi        = psi        + DTee*erk_b(s)*fpsi(:,:,s);
                 Pi         = Pi         + DTee*erk_b(s)*fPi(:,:,s);
                 Phi        = Phi        + DTee*erk_b(s)*fPhi(:,:,s);
                 Hstarpsi   = Hstarpsi   + DTee*erk_b(s)*fHstarpsi(:,s);
                 Fstarpsi   = Fstarpsi   + DTee*erk_b(s)*fFstarpsi(:,s);
                 Gstarpsi   = Gstarpsi   + DTee*erk_b(s)*fGstarpsi(:,s);
                 Hstarpsi_E = Hstarpsi_E + DTee*erk_b(s)*fHstarpsi_E(:,s);
                 Fstarpsi_E = Fstarpsi_E + DTee*erk_b(s)*fFstarpsi_E(:,s);
                 Gstarpsi_E = Gstarpsi_E + DTee*erk_b(s)*fGstarpsi_E(:,s);
             end
             Tee = j*DTee;
             %
             %  Record output Waveform.
             %
             r_record = 20.0*mass;
             x_record = r_record + log(r_record/(2*mass)-1);
             for q = 1:ndmn
                 if x_record >= c(q) & x_record <= c(q+1)
                    psi_record = chbevalv(real(psi(:,q)), ...
                                     x_record,c(q),c(q+1));
                    break
                 end
             end
%
%  Commented out lines here are for using the same convolutions for the
%  extraction as for the boundary conditions.
%
%               psi_E = sum(mu_E.*Hstarpsi) ...
%                    +2*sum(mR_E.*Gstarpsi) ...
%                    -2*sum(mI_E.*Fstarpsi) ...
%                    +psi(npts,ndmn);
                psi_E = sum(mu_E.*Hstarpsi_E) ...
                     +2*sum(mR_E.*Gstarpsi_E) ...
                     -2*sum(mI_E.*Fstarpsi_E) ...
                     +psi(npts,ndmn);
                output(j+1,:) = [Tee, psi_record, psi_E, psi(npts,ndmn)];
             %
             %  Test for blowup.
             %
             blowup = 20;
             psimax= max(max(abs(psi)));
             if psimax > blowup
               disp(['-----------------------------------------'])
               disp([' '])
               disp(['Quitting, solution > ',num2str(blowup)])
               disp([' ',num2str(Tee),' ',num2str(psimax)])
               disp(['-----------------------------------------'])
               return
             end
             if mod(Tee,printTee) <= DTee && Tee > 2*DTee
                elapsed = toc/60;
                total_est = MAXTIME*elapsed/Tee;
                remain_est = total_est - elapsed;
                disp(['-----------------------------------------'])
                disp([' '])
                disp(['Program time       : ',num2str(round(Tee))])
                disp(['Elapsed clock  (m) : ',num2str(elapsed)])
                disp(['Time remaining (m) : ',num2str(remain_est)])
                disp([' '])
                disp(['-----------------------------------------'])
             end
         end
         disp(['Elapsed time (s)   : ',num2str(toc)])
         disp(['Tee                : ',num2str(Tee)])
         disp(['MAXTIME            : ',num2str(MAXTIME)])
         if makePlots == 1
         for i = 1:length(x0);
             for q = 1:ndmn
                 if x0(i) >= c(q) & x0(i) <= c(q+1)
                    psi0(i) = chbevalv(real(psi(:,q)), ...
                                     x0(i),c(q),c(q+1));
                    Pi0(i)  = chbevalv(real(Pi(:,q)), ...
                                     x0(i),c(q),c(q+1));
                    Phi0(i) = chbevalv(real(Phi(:,q)), ...
                                     x0(i),c(q),c(q+1));
                    break
                 end
             end
         end
%
%  Plot computed psi, Pi, Phi.
%
         figure(2)
         subplot(3,1,1)
         hold on
         plot(x0,psi0,'k','LineWidth',2)
         set(gca, 'FontSize', 16, 'fontweight', 'n')
         legend('Final','Location','NorthWest')
         title('Solution $\psi$','interpreter','latex')
         subplot(3,1,2)
         hold on
         plot(x0,Pi0,'k','LineWidth',2)
         set(gca, 'FontSize', 16, 'fontweight', 'n')
         legend('Final','Location','NorthWest')
         title('Solution $\Pi$','interpreter','latex')
         subplot(3,1,3)
         hold on
         plot(x0,Phi0,'k','LineWidth',2)
         set(gca, 'FontSize', 16, 'fontweight', 'n')
         legend('Final','Location','NorthWest')
         title('Solution $\Phi$','interpreter','latex')
         xlabel('$x$','interpreter','latex')
         pause(2)
         end
         if qexp == 1
            psilast = psi0;
            Pilast  = Pi0;
            Philast = Phi0;
            switch model
            case 0,
               save 'outputRW.dat' -ascii -double output
            case 1,
               save 'outputCPM.dat' -ascii -double output
            case 2,
               save 'outputZM.dat' -ascii -double output
            otherwise,
               error = 'model not 0 or 1 or 2.'
            end
        else
            ln    = length(psi0);
            Dpsi  = abs(psi0-psilast);
            DPi   = abs(Pi0-Pilast);
            DPhi  = abs(Phi0-Philast);
            L1E = (sum(Dpsi)+sum(DPi)+sum(DPhi))/(3*ln);
            L2E = sqrt(sum(Dpsi.^2)+sum(DPi.^2)+sum(DPhi.^2))/(3*ln);
            LiE = max(max([Dpsi;DPi;DPhi]));
            error_1_2_i(qexp-1,:) = [L1E, L2E, LiE];
            plotDt(qexp-1)    = DTee;
            plotErr(qexp-1,1) = L1E;
            plotErr(qexp-1,2) = L2E;
            plotErr(qexp-1,3) = LiE;
            figure(3)
            subplot(3,1,1)
            hold off
            plot(x0,psi0-psilast,'-k.','LineWidth',2,'MarkerSize',8)
            set(gca, 'FontSize', 16, 'fontweight', 'n')
            title('error $\psi \mbox{-}\!\mbox{-} \psi_\mathrm{last}$','interpreter','latex')
            subplot(3,1,2);
            hold off;
            plot(x0,Pi0-Pilast,'-k.','LineWidth',2,'MarkerSize',8)
            set(gca, 'FontSize', 16, 'fontweight', 'n')
            title('error $\Pi \mbox{-}\!\mbox{-} \Pi_\mathrm{last}$','interpreter','latex')
            subplot(3,1,3)
            hold off
            plot(x0,Phi0-Philast,'-k.','LineWidth',2,'MarkerSize',8)
            set(gca, 'FontSize', 16, 'fontweight', 'n');
            title('error $\Phi \mbox{-}\!\mbox{-} \Phi_\mathrm{last}$','interpreter','latex')
            xlabel('$x$','interpreter','latex')
            psilast = psi0;
            Pilast = Pi0;
            Philast = Phi0;
            pause(4)
         end
      end
      if length(exponents) < 2
         return
      end
%
%  Compute a slope for the convergence rate. Assumption is
%  Err = cst * (Dt)^rate, so logErr = rate * logDt + logcst.
%  The command polyfit, with trailing argument 1, computes
%  the best line (rate, logcst), in the least squares sense,
%  through the specified data.
%
      finish = length(plotDt);
      start =  max(1,finish-2);
      start = 1;
      poly   = polyfit(log(plotDt(start:finish)),...
                   log(plotErr(start:finish,3)),1);
      rate = poly(1);
      cst = plotErr(start,3)/plotDt(start)^rate;
      figure(4)
      hold off
      loglog(plotDt,cst*plotDt.^rate,'--k','LineWidth',2,'MarkerSize',8)
      set(gca, 'FontSize', 16, 'fontweight', 'n')
      legend(['rate (from least squares) = ',num2str(rate)])
      hold on
%
%  Now plot the actual error vs time step on loglog plot.
%
      loglog(plotDt,plotErr(:,3),'bo','LineWidth',2,'MarkerSize',8)
      set(gca, 'FontSize', 16, 'fontweight', 'n')
      title('Error vs time-step','interpreter','latex')
      xlabel('time-step $\Delta t$','interpreter','latex')
      ylabel('$L_\infty$ error','interpreter','latex')
