%**** FUNCTION REGGEWHEELERV.M *****************************
%                                                          *
%  S R Lau                                                 *
%                                                          *
%  Applied Mathematics Group                  1998 to 2004 *
%  University of North Carolina                            *
%  Chapel Hill, NC 27599                                   *
%                                                          *
%  Center for Gravitational Wave Astronomy    2004 to 2005 *
%  University of Texas at Brownsville                      *
%  Brownsville, TX 78520                                   *
%                                                          *
%  Mathematics & Statistics                   2005 to 2006 *
%  University of New Mexico                                *
%  Albuquerque, NM 87131                                   *
%                                                          *
%  Division of Applied Mathematics            2006 to ???? *
%  Brown University                                        *
%  Providence, RI 02906                                    *
%                                                          *
%  Last modified: 26 September 2009                        *
%                                                          *
%***********************************************************
      function [V]=ReggeWheelerV(x,M,ell,spin)
      if ell < 2
         if ell == 0
            if spin ~= 0
               error = 'ell = 0 but spin not 0.'
               pause
            end
         else
            if spin ~= 0 && spin ~ 1
               error = 'ell = 1 but spin not 1 or 0.'
               pause
            end
         end
      end 
      V = zeros(size(x));
      %
      %  Should work for both linear an block array 
      %  structure for x.
      %
      [dim1 dim2] = size(x);
      LENGTHx = dim1*dim2;
      twoM   = 2*M;
      for k = 1:LENGTHx
          xk = x(k);
          if xk/twoM < -700
             %
             %  Range experimentially verified
             %  by hand in Matlab.
             %
             error = 'Tortoise coordinate too small.'
             pause
          end
          if xk > 4.0*M
             rk    = xk;
             rm2Mk = rk-twoM;
          else
             rm2Mk = twoM*exp(-1+xk/twoM);
             rk    = rm2Mk+twoM;
          end
          for i=1:100
             err = rm2Mk+twoM*(1+log(rm2Mk/twoM))-xk;
             if abs(err) < 1.0e-25
                break
             end
             %
             %  With x(r) = r + 2M log[-1+r/(2M)], the
             %  function to zero is F(r) = x(r) - xk,
             %  and dF/dr = dx/dr. Newton iteration is
             %  rnew = r-F/(dF/dr) = r-[x(r)-xk]/(dx/dr).
             %  Subtracting 2M from each side, we get
             %  (r-2M)new = (r-2M)-[x(r)-xk]/(dx/dr).
             %
             dxdr = (rm2Mk+twoM)/rm2Mk;
             rm2M_temp = rm2Mk-err/dxdr;
             %
             %  Previous two lines work, but can reduce
             %  expression to get following (also works,
             %  but above lines slightly more efficient).
             %
             %  rm2M_temp = rm2Mk*(xk ...
             %  -twoM*log(rm2Mk/twoM) ...
             %            )/(rm2Mk+twoM);
             %
             %  Make sure rm2Mk stays positive.
             %
             if rm2M_temp < 0
                rm2M_temp = 0.5*rm2Mk;
             end
             rm2Mk  = rm2M_temp;
             rk     = rm2Mk+twoM;
         end
         rk3  = rk*rk*rk;
         V(k) = rm2Mk*(ell*(ell+1) ...
              + twoM*(1-spin*spin)/rk);
         V(k) = V(k)/rk3;
      end
